package com.yc.ship.module.otc.dal.dataobject.storetradelog;

import com.baomidou.mybatisplus.annotation.KeySequence;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.sun.xml.bind.v2.TODO;
import com.yc.ship.framework.tenant.core.db.TenantBaseDO;
import lombok.*;

import java.math.BigDecimal;

/**
 * 旅行社门店交易记录：余额充值、预付购票、预付退款、余额购票 DO
 *
 * @author 管理员
 */
@TableName("otc_agency_store_trade_log")
@KeySequence("otc_agency_store_trade_log_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class StoreTradeLogDO extends TenantBaseDO {

    /**
     * 交易记录ID
     */
    @TableId
    private Long id;
    /**
     * 门店ID
     */
    private Long storeId;
    /**
     * 交易名称
     */
    private String tradeName;
    /**
     * 交易类型：1.管理员余额充值、2.管理员授信充值、3.余额购票、4.授信购票、5.余额退款、6.授信退款
     * <p>
     * 枚举 {@link TODO trade_type 对应的类}
     */
    private Integer tradeType;
    /**
     * 交易前余额/交易前已用额度
     */
    private BigDecimal beforeBalance;
    /**
     * 交易后余额/交易后已用额度
     */
    private BigDecimal afterBalance;
    /**
     * 交易金额
     */
    private BigDecimal tradeAmount;
    /**
     * 交易类型 1.增加、2.减少
     * <p>
     * 枚举 {@link TODO amount_type 对应的类}
     */
    private Integer tradeMode;
    /**
     * 备注
     */
    private String remark;
    /**
     * 订单号
     */
    private String orderId;
    /**
     * 退单号
     */
    private String refundId;

}
