package com.yc.ship.module.resource.service.room;

import java.util.*;
import javax.validation.*;
import com.yc.ship.module.resource.controller.admin.room.vo.*;
import com.yc.ship.module.resource.dal.dataobject.room.ResourceRoomDO;
import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.framework.common.pojo.PageParam;

/**
 * 资源管理-游轮房间管理 Service 接口
 *
 * @author flycloud
 */
public interface ResourceRoomService {

    /**
     * 创建资源管理-游轮房间管理
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createRoom(@Valid ResourceRoomSaveReqVO createReqVO);

    /**
     * 更新资源管理-游轮房间管理
     *
     * @param updateReqVO 更新信息
     */
    void updateRoom(@Valid ResourceRoomSaveReqVO updateReqVO);

    /**
     * 删除资源管理-游轮房间管理
     *
     * @param id 编号
     */
    void deleteRoom(Long id);

    /**
     * 获得资源管理-游轮房间管理
     *
     * @param id 编号
     * @return 资源管理-游轮房间管理
     */
    ResourceRoomDO getRoom(Long id);

    /**
     * 获得资源管理-游轮房间管理分页
     *
     * @param pageReqVO 分页查询
     * @return 资源管理-游轮房间管理分页
     */
    PageResult<ResourceRoomDO> getRoomPage(ResourceRoomPageReqVO pageReqVO);

    /**
     * 根据游轮ID获取房间列表
     * @param shipId
     * @return
     */
    List<ResourceRoomDO> getRoomListByShipId(String shipId);

    /**
     * 根据游轮ID和楼层获取房间列表
     * @param shipId
     * @param floors
     * @return
     */
    List<ResourceRoomDO> getRoomListByShipIdAndFloors(String shipId, String floors);
}